@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2012 -- leonerd@leonerd.org.uk
 
 package IO::Async::Process;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.44';
+our $VERSION = '0.46';
 
 use Carp;
 
@@ -173,7 +173,8 @@ from the other.
 =item pipe_write
 
 The child will be given the reading end of a C<pipe(2)>; the parent may write
-to the other.
+to the other. Since an EOF condition of this kind of handle cannot reliably be
+detected, C<on_finish> will not wait for this type of pipe to be closed.
 
 =item pipe_rdwr
 
@@ -206,7 +207,7 @@ wrapped by an C<IO::Async::Stream> using this C<on_read> callback function.
 
 =item into => SCALAR
 
-The child will be given the reading end of a pipe. The referenced scalar will
+The child will be given the writing end of a pipe. The referenced scalar will
 be filled by data read from the child process. This data may not be available
 until the pipe has been closed by the child.
 
@@ -389,6 +390,7 @@ sub _prepare_fds
       my $handle = $self->fd( $fd );
 
       my $key = $fd eq "io" ? "stdio" : "fd$fd";
+      my $write_only;
 
       if( $via == FD_VIA_PIPEREAD ) {
          my ( $myfd, $childfd ) = $loop->pipepair or croak "Unable to pipe() - $!";
@@ -400,6 +402,7 @@ sub _prepare_fds
       }
       elsif( $via == FD_VIA_PIPEWRITE ) {
          my ( $childfd, $myfd ) = $loop->pipepair or croak "Unable to pipe() - $!";
+         $write_only++;
 
          $handle->configure( write_handle => $myfd );
 
@@ -436,12 +439,14 @@ sub _prepare_fds
          croak "Unsure what to do with fd_via==$via";
       }
 
-      $mergepoint->needs( $key );
-      $handle->configure(
-         on_closed => sub {
-            $mergepoint->done( $key );
-         },
-      );
+      unless( $write_only ) {
+         $mergepoint->needs( $key );
+         $handle->configure(
+            on_closed => sub {
+               $mergepoint->done( $key );
+            },
+         );
+      }
 
       $self->add_child( $handle );
    }
@@ -457,6 +462,9 @@ sub _add_to_loop
    $self->{code} or $self->{command} or
       croak "Require either 'code' or 'command' in $self";
 
+   $self->can_event( "on_finish" ) or
+      croak "Expected either an on_finish callback or to be able to ->on_finish";
+
    my @setup;
    push @setup, @{ $self->{setup} } if $self->{setup};
 
@@ -490,7 +498,8 @@ sub _add_to_loop
    my $is_code = defined $self->{code};
 
    $mergepoint->close(
-      on_finished => sub {
+      on_finished => $self->_capture_weakself( sub {
+         my $self = shift or return;
          my %items = @_;
 
          $self->{exitcode} = $exitcode;
@@ -508,8 +517,8 @@ sub _add_to_loop
                $self->invoke_event( on_finish => $exitcode );
          }
 
-         $self->_remove_from_outer;
-      },
+         $self->remove_from_parent;
+      } ),
    );
 }
 
@@ -543,6 +552,20 @@ sub pid
    return $self->{pid};
 }
 
+=head2 $process->kill( $signal )
+
+Sends a signal to the process
+
+=cut
+
+sub kill
+{
+   my $self = shift;
+   my ( $signal ) = @_;
+
+   kill $signal, $self->pid or croak "Cannot kill() - $!";
+}
+
 =head2 $running = $process->is_running
 
 Returns true if the Process has been started, and has not yet finished.
@@ -697,6 +720,117 @@ sub stdout { shift->fd( 1 ) }
 sub stderr { shift->fd( 2 ) }
 sub stdio  { shift->fd( 'io' ) }
 
+=head1 EXAMPLES
+
+=head2 Capturing the STDOUT stream of a process
+
+By configuring the C<stdout> filehandle of the process using the C<into> key,
+data written by the process can be captured.
+
+ my $stdout;
+ my $process = IO::Async::Process->new(
+    command => [ "writing-program", "arguments" ],
+    stdout => { into => \$stdout },
+    on_finish => sub {
+       print "The process has finished, and wrote:\n";
+       print $stdout;
+    }
+ );
+
+ $loop->add( $process );
+
+Note that until C<on_finish> is invoked, no guarantees are made about how much
+of the data actually written by the process is yet in the C<$stdout> scalar.
+
+See also the C<run_child> method of L<IO::Async::Loop>.
+
+To handle data more interactively as it arrives, the C<on_read> key can
+instead be used, to provide a callback function to invoke whenever more data
+is available from the process.
+
+ my $process = IO::Async::Process->new(
+    command => [ "writing-program", "arguments" ],
+    stdout => {
+       on_read => sub {
+          my ( $stream, $buffref ) = @_;
+          while( $$buffref =~ s/^(.*)\n// ) {
+             print "The process wrote a line: $1\n";
+          }
+
+          return 0;
+       },
+    },
+    on_finish => sub {
+       print "The process has finished\n";
+    }
+ );
+
+ $loop->add( $process );
+
+If the code to handle data read from the process isn't available yet when
+the object is constructed, it can be supplied later by using the C<configure>
+method on the C<stdout> filestream at some point before it gets added to the
+Loop. In this case, C<stdin> should be configured using C<pipe_read> in the
+C<via> key.
+
+ my $process = IO::Async::Process->new(
+    command => [ "writing-program", "arguments" ],
+    stdout => { via => "pipe_read" },
+    on_finish => sub {
+       print "The process has finished\n";
+    }
+ );
+
+ $process->stdout->configure(
+    on_read => sub {
+       my ( $stream, $buffref ) = @_;
+       while( $$buffref =~ s/^(.*)\n// ) {
+          print "The process wrote a line: $1\n";
+       }
+
+       return 0;
+    },
+ );
+
+ $loop->add( $process );
+
+=head2 Sending data to STDIN of a process
+
+By configuring the C<stdin> filehandle of the process using the C<from> key,
+data can be written into the C<STDIN> stream of the process.
+
+ my $process = IO::Async::Process->new(
+    command => [ "reading-program", "arguments" ],
+    stdin => { from => "Here is the data to send\n" },
+    on_finish => sub { 
+       print "The process has finished\n";
+    }
+ );
+
+ $loop->add( $process );
+
+The data in this scalar will be written until it is all consumed, then the
+handle will be closed. This may be useful if the program waits for EOF on
+C<STDIN> before it exits.
+
+To have the ability to write more data into the process once it has started.
+the C<write> method on the C<stdin> stream can be used, when it is configured
+using the C<pipe_write> value for C<via>:
+
+ my $process = IO::Async::Process->new(
+    command => [ "reading-program", "arguments" ],
+    stdin => { via => "pipe_write" },
+    on_finish => sub { 
+       print "The process has finished\n";
+    }
+ );
+
+ $loop->add( $process );
+
+ $process->stdin->write( "Here is some more data\n" );
+
+=cut
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
